import requests

from posixpath import join as posixjoin

from sermonaudio import BASE_URL, get_request_headers
from sermonaudio.node import types

URL_PATH = 'node'

def _get_sermons_node(node_name: str, params: dict, preferred_language_override=None):
    """Calls an endpoint named node_name with params, and expects to get a list of sermons

    :param node_name: The name of the node to call
    :param params: A dictionary of query string parameters
    :param preferred_language_override: A preferred langauge code, if you don't want to use your system locale
    :rtype: [sermonaudio.types.Sermon]
    """
    url = posixjoin(BASE_URL, URL_PATH, node_name)
    response = requests.get(url, params=params, headers=get_request_headers(preferred_language_override))

    node = types.Node(response.json())
    assert node.node_type == node_name

    sermons = [types.Sermon(result) for result in node.results]
    for sermon in sermons:
        assert isinstance(sermon, types.Sermon)

    return sermons


def get_sermon_info(sermon_id: str, preferred_language_override=None):
    """Calls the sermon_info node endpoint.

    :param sermon_id: The ID of the sermon you want to fetch
    :param preferred_language_override: An optional override to the Accept-Language header for a single request
    :rtype: sermonaudio.types.Sermon or None
    """
    node_name = 'sermon_info'
    params = {
        'sermonID': sermon_id
    }
    sermons = _get_sermons_node(node_name, params, preferred_language_override)

    if len(sermons) > 0:
        return sermons[0]
    else:
        return None


def get_newest_audio_sermons(page: int = 1, preferred_language_override=None):
    """Calls the newest_audio_sermons node endpoint.

    Returned sermons are sorted by media upload date descending.

    :param page: The page number to load (defaults to 1).
    :param preferred_language_override: An optional override to the Accept-Language header for a single request
    :rtype: [sermonaudio.types.Sermon]
    """
    node_name = 'newest_audio_sermons'
    params = {
        'page': page
    }

    return _get_sermons_node(node_name, params, preferred_language_override)


def get_newest_video_sermons(page: int = 1, preferred_language_override=None):
    """Calls the newest_video_sermons node endpoint.

    Returned sermons are sorted by media upload date descending.

    :param page: The page number to load (defaults to 1)
    :param preferred_language_override: An optional override to the Accept-Language header for a single request
    :rtype: [sermonaudio.types.Sermon]
    """
    node_name = 'newest_video_sermons'
    params = {
        'page': page
    }

    return _get_sermons_node(node_name, params, preferred_language_override)


def get_sermons_by_source(source_id: str, page: int = 1, preferred_language_override=None):
    """Calls the sermons_by_source node endpoint.

    Returned sermons are sorted by date descending.

    :param source_id: The ID of the source you want a listing from
    :param page: The page number to load (defaults to 1)
    :param preferred_language_override: An optional override to the Accept-Language header for a single request
    :rtype: [sermonaudio.types.Sermon]
    """
    node_name = 'sermons_by_source'
    params = {
        'sourceID': source_id,
        'page': page
    }

    return _get_sermons_node(node_name, params, preferred_language_override)


def get_sermons_by_speaker(speaker_name: str, preferred_language_override=None):
    """Calls the sermons_by_speaker node endpoint.

    Returned sermons are sorted by date descending.

    :param speaker_name: The name of the speaker
    :param preferred_language_override: An optional override to the Accept-Language header for a single request
    :rtype: list of sermonaudio.types.Sermon
    """
    node_name = 'sermons_by_speaker'
    url = posixjoin(BASE_URL, URL_PATH, node_name)
    params = {
        'speakerName': speaker_name
    }

    return _get_sermons_node(node_name, params, preferred_language_override)


def get_sermons_by_language(language_code: str, page: int = 1, preferred_language_override=None):
    """Calls the sermons_by_language node endpoint.

    Returned sermons are sorted by date descending.

    :param language_code: An ISO 639 language code
    :param page: The page number to load (defaults to 1)
    :param preferred_language_override: An optional override to the Accept-Language header for a single request
    :rtype: [sermonaudio.types.Sermon]
    """
    node_name = 'sermons_by_language'
    params = {
        'languageCode': language_code,
        'page': page
    }

    return _get_sermons_node(node_name, params, preferred_language_override)


def get_source_info(source_id: str, preferred_language_override=None):
    """Calls the source_info node endpoint.

    :param source_id: The ID of the source you want info on.
    :param preferred_language_override: An optional override to the Accept-Language header for a single request
    :rtype: sermonaudio.types.Source
    """
    node_name = 'source_info'
    url = posixjoin(BASE_URL, URL_PATH, node_name)
    params = {
        'sourceID': source_id
    }

    response = requests.get(url, params=params, headers=get_request_headers(preferred_language_override))

    node = types.Node(response.json())
    assert node.node_type == node_name

    if len(node.results) > 0:
        source = types.Source(node.results[0])
        assert isinstance(source, types.Source)

        return source
    else:
        return None


def get_webcasts_in_progress(preferred_language_override=None):
    """Calls the webcasts_in_progress node endpoint.

    :param preferred_language_override: An optional override to the Accept-Language header for a single request
    :rtype: [sermonaudio.types.WebcastInfo]
    """
    node_name = 'webcasts_in_progress'
    url = posixjoin(BASE_URL, URL_PATH, node_name)

    response = requests.get(url, headers=get_request_headers(preferred_language_override))

    node = types.Node(response.json())
    assert node.node_type == node_name

    webcasts = [types.WebcastInfo(result) for result in node.results]

    for webcast in webcasts:
        assert isinstance(webcast, types.WebcastInfo)

    return webcasts


def get_sources_near_location(latitude: float, longitude: float, meters: int, preferred_language_override=None):
    """Calls the source_info node endpoint.

    :param latitude: The latitude of the search origin
    :param longitude: The longitude of the search origin
    :param meters: THe distance from the origin to search
    :param preferred_language_override: An optional override to the Accept-Language header for a single request
    :rtype: sermonaudio.types.Source
    """
    node_name = 'sources_near_location'
    url = posixjoin(BASE_URL, URL_PATH, node_name)
    params = {
        'latitude': latitude,
        'longitude': longitude,
        'meters': meters
    }

    response = requests.get(url, params=params, headers=get_request_headers(preferred_language_override))

    node = types.Node(response.json())
    assert node.node_type == node_name

    sources_near_location = [types.SourceNearLocation(result) for result in node.results]

    for source_near_location in sources_near_location:
        assert isinstance(source_near_location, types.SourceNearLocation)

    return sources_near_location



from .requests import get_sermon_info, get_newest_audio_sermons, get_newest_video_sermons, get_sermons_by_source, \
    get_sermons_by_speaker, get_sermons_by_language, get_source_info, get_webcasts_in_progress, \
    get_sources_near_location



"""Node API data structures"""
import datetime
import pytz


def isinstance_or_none(obj, type):
    return (obj is None) or isinstance(obj, type)


class Node(object):
    """The base node object, which encapsulates all node API responses"""
    def __init__(self, obj: dict):
        self.node_type = obj['nodeType']
        assert isinstance(self.node_type, str)

        self.node_display_name = obj['nodeDisplayName']
        assert isinstance(self.node_display_name, str)

        self.results = obj['results']
        assert isinstance(self.results, list)


class Source(object):
    """Broadcaster API data model"""
    def __init__(self, obj: dict):
        """Constructs a broadcaster from a dictionary returned from the SermonAudio API"""
        self.source_id = obj['sourceID']
        assert isinstance(self.source_id, str)

        self.service_times_are_preformatted = obj['serviceTimesArePreformatted']
        assert isinstance_or_none(self.service_times_are_preformatted, bool)

        self.service_times = obj['serviceTimes']
        assert isinstance_or_none(self.service_times, str)

        self.denomination = obj['denomination']
        assert isinstance_or_none(self.denomination, str)

        self.address = obj['address']
        assert isinstance_or_none(self.address, str)

        self.display_name = obj['displayName']
        assert isinstance(self.display_name, str)

        self.location = obj['location']
        assert isinstance(self.location, str)

        self.latitude = obj['latitude']
        assert isinstance_or_none(self.latitude, float)

        self.longitude = obj['longitude']
        assert isinstance_or_none(self.longitude, float)

        self.image_url = obj['imageURL']
        assert isinstance(self.image_url, str)

        self.album_art_url_format = obj['albumArtURL']
        assert isinstance(self.album_art_url_format, str)

        self.minister = obj['minister']
        assert isinstance_or_none(self.minister, str)

        self.phone = obj['phone']
        assert isinstance_or_none(self.phone, str)

        self.home_page_url = obj['homePageURL']
        assert isinstance_or_none(self.home_page_url, str)

        self.bible_version = obj['bibleVersion']
        assert isinstance_or_none(self.bible_version, str)

        self.facebook_username = obj['facebookUsername']
        assert isinstance_or_none(self.facebook_username, str)

        self.twitter_username = obj['twitterUsername']
        assert isinstance_or_none(self.twitter_username, str)

        self.about_us = obj['aboutUs']
        assert isinstance_or_none(self.about_us, str)

        self.disabled = obj['disabled']
        assert isinstance(self.disabled, bool)

    def get_album_art_url(self, size: int):
        """Returns a URL for the square album art with a with and height equal to the provided size argument"""
        return self.album_art_url_format.replace('{size}', str(size))


class Speaker(object):
    """Speaker data model"""
    def __init__(self, obj: dict):
        self.display_name = obj['displayName']
        assert isinstance(self.display_name, str)

        self.bio = obj['bio']
        assert isinstance_or_none(self.bio, str)

        self.portrait_url = obj['portraitURL']
        assert isinstance(self.portrait_url, str)

        self.rounded_thumbnail_image_url = obj['roundedThumbnailImageURL']
        assert isinstance(self.rounded_thumbnail_image_url, str)

        self.album_art_url_format = obj['albumArtURL']
        assert isinstance(self.album_art_url_format, str)

    def get_album_art_url(self, size: int):
        """Returns a URL for the square album art with a with and height equal to the provided size argument"""
        return self.album_art_url_format.replace('{size}', str(size))


class Sermon(object):
    """Sermon data model"""
    def __init__(self, obj: dict):
        """Constructs a sermon from a dictionary returned from the SermonAudio API"""
        self.sermon_id = obj['sermonID']
        assert isinstance(self.sermon_id, str)

        self.source = Source(obj['source'])
        self.speaker = Speaker(obj['speaker'])

        self.audio_duration_in_seconds = obj['audioDurationInSeconds']
        assert isinstance_or_none(self.audio_duration_in_seconds, int)

        self.full_title = obj['fullTitle']
        assert isinstance(self.full_title, str)

        self.display_title = obj['displayTitle']
        assert isinstance(self.display_title, str)

        self.subtitle = obj['subtitle']
        assert isinstance_or_none(self.subtitle, str)

        self.preach_date = datetime.datetime.strptime(obj['preachDate'], '%Y-%m-%d').date()
        assert isinstance(self.preach_date, datetime.date)

        self.language_code = obj['languageCode']
        assert isinstance(self.language_code, str)

        self.bible_text = obj['bibleText']
        assert isinstance_or_none(self.bible_text, str)

        self.more_info_text = obj['moreInfoText']
        assert isinstance_or_none(self.more_info_text, str)

        self.event_type = obj['eventType']
        assert isinstance(self.event_type, str)

        self.download_count = obj['downloadCount']
        assert isinstance(self.download_count, int)

        self.audio_bitrate = obj['audioBitrate']
        assert isinstance_or_none(self.audio_bitrate, int)

        self.video_bitrate = obj['videoBitrate']
        assert isinstance_or_none(self.video_bitrate, int)

        self.audio_file_url = obj['audioFileURL']
        assert isinstance_or_none(self.audio_file_url, str)

        self.audio_file_lo_url = obj['audioFileLOURL']
        assert isinstance_or_none(self.audio_file_lo_url, str)

        self.video_file_url = obj['videoFileURL']
        assert isinstance_or_none(self.video_file_url, str)

        self.video_file_lo_url = obj['videoFileLOURL']
        assert isinstance_or_none(self.video_file_lo_url, str)

        self.video_hls_url = obj['videoHLSURL']
        assert isinstance_or_none(self.video_hls_url, str)

        self.video_thumbnail_image_url = obj['videoThumbnailImageURL']
        assert isinstance_or_none(self.video_thumbnail_image_url, str)

        self.text_file_url = obj['textFileURL']
        assert isinstance_or_none(self.text_file_url, str)

    def __str__(self):
        return '{0:s} - {1:s}'.format(self.speaker.display_name, self.display_title)


class HLSStreamInfo(object):
    """An HLS video stream info object"""

    def __init__(self, bitrate: int, url: str):
        """Constructs an HLS stream info object"""
        self.bitrate = bitrate
        assert isinstance(self.bitrate, int)

        self.url = url
        assert isinstance(self.url, str)


class WebcastInfo(object):
    """Webcast info data model"""

    def __init__(self, obj: dict):
        """Constructs webcast info from a dictionary returned from the SermonAudio API"""
        self.display_name = obj['displayName']
        assert isinstance(self.display_name, str)

        self.source_id = obj['sourceID']
        assert isinstance(self.source_id, str)

        self.source_location = obj['sourceLocation']
        assert isinstance(self.source_location, str)

        self.start_time = datetime.datetime.fromtimestamp(obj['startTime'], tz=pytz.utc)
        assert isinstance(self.start_time, datetime.datetime)

        self.preview_image_url = obj['previewImageURL']
        assert isinstance_or_none(self.preview_image_url, str)

        self.peak_listener_count = obj['peakListenerCount']
        assert isinstance(self.peak_listener_count, int)

        self.total_tune_in_count = obj['totalTuneInCount']
        assert isinstance(self.total_tune_in_count, int)

        self.has_video = obj['hasVideo']
        assert isinstance(self.has_video, bool)

        self.audio_url = obj['audioURL']
        assert isinstance(self.audio_url, str)

        self.hls_video_streams = []
        for stream in obj['hlsVideoStreams']:
            stream_info = HLSStreamInfo(stream['bitrate'], stream['url'])
            self.hls_video_streams.append(stream_info)


class SourceNearLocation(object):
    """Model representing a source and it's distance from a search origin"""

    def __init__(self, obj: dict):
        """Constructs source near location from a dictionary returned from the SermonAudio API"""
        self.source = Source(obj['source'])
        assert isinstance(self.source, Source)

        self.meters = obj['meters']
        assert isinstance(self.meters, int)



import ssl
import locale

from requests import Session
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.poolmanager import PoolManager

_api_key = None
_preferred_language = (locale.getdefaultlocale()[0] or 'en-US').replace('_', '-')

BASE_URL = "https://api.sermonaudio.com/v1/"


def set_api_key(key: str):
    """Sets the SermonAudio API key to use for authenticated requests. A key must be set before making any requests.

    :param key: Your API key. Broadcasters may obtain an API key by going to the Member's Only area of the site. If you
    are an application developer and would like to integrate SermonAudio into your product or website, please contact
    us at info@sermonaudio.com and we'll be happy to discuss with you.
    """
    global _api_key
    _api_key = key


def set_preferred_language(http_accept_language: str):
    """Sets the preferred language string globally. Defaults to your system locale environment variables.

    The format of the string should be an acceptable HTTP Accept-Language header. You may specify multiple languages,
    and the API will attempt to accommodate your preferences when possible. While content language will always be
    determined by the broadcasters individually, other strings, whenever possible, will be translated."""
    global _preferred_language
    _preferred_language = http_accept_language


def get_request_headers(preferred_language_override=None, show_content_in_any_language=True):
    result = {
        'X-API-Key': _api_key,
        'Accept-Language': preferred_language_override or _preferred_language
    }

    if show_content_in_any_language:
        result['X-Show-All-Languages'] = 'True'

    return result


class SecureRequestAdapter(HTTPAdapter):
    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = PoolManager(num_pools=connections,
                                       maxsize=maxsize,
                                       block=block,
                                       ssl_version=ssl.PROTOCOL_TLSv1_2)


# Require TLSv1.2 for all connections to the server
s = Session()
s.mount(BASE_URL, SecureRequestAdapter())



from setuptools import setup

try:
    import pypandoc
    readme = pypandoc.convert('README.md', 'rst', outputfile='README.rst')
except ImportError:
    print('WARNING: Package being build without ReST documentation. Please install pypandoc.')
    readme = None

setup(name='sermonaudio',
      version='0.1.4',
      description='The official Python client library for accessing the SermonAudio.com APIs',
      long_description=readme,
      classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 3.5',
      ],
      url='http://api.sermonaudio.com/',
      author='SermonAudio.com',
      author_email='info@sermonaudio.com',
      keywords='sermon audio sermonaudio API preaching church bible',
      license='MIT',
      packages=['sermonaudio', 'sermonaudio.node'],
      install_requires=[
            'requests>=2.9',
            'pytz>=2016.3'
      ])
